package ru.ifmo.vizi.dmkm;


import java.util.StringTokenizer;
import java.io.*;
import ru.ifmo.vizi.base.SmartTokenizer;
import ru.ifmo.vizi.base.Base;
import ru.ifmo.vizi.base.I18n;
import ru.ifmo.vizi.base.VisualizerParameters;
import ru.ifmo.vizi.base.ui.SaveLoadDialog;
import ru.ifmo.vizi.base.ui.AutoControlsPane;
import ru.ifmo.vizi.base.ui.HintedButton;
import ru.ifmo.vizi.base.ui.MultiButton;
import ru.ifmo.vizi.base.ui.SpinPanel;
import ru.ifmo.vizi.dmkm.net.Net;
import ru.ifmo.vizi.dmkm.net.RandomNetGenerator;


import java.awt.*;
import java.awt.*;

/**
 * Applet.
 *
 * @author Yuri Bedny
 * @version $Id$
 */

public final class DMKMVisualizer extends Base {

    private Frame forefather;

    private final DMKM auto;

    private final DMKM.Data data;

    private Net net;

    private NetEditor editor;

    private AutoControlsPane controlsPane;

    private Panel panel;

    private Panel editPanel;

    private MultiButton modeButton;

    private SaveLoadDialog saveLoadDialog;
	private	HintedButton saveLoadButton;
	
    private NetRecord lNets[];

    private SpinPanel nPanel;

    private Panel bottomPanel;


    /**
     * Rewinds algorithm to the specified step.
     *
     * @param step spet of the algorith to rewind to.
     */

    private void rewind(int n, int g[][], int step) {
    	clearNet();
        data.n = n;
        data.t = n - 1;
        net.setNumOfVertices(n);
        allocateMem();
        setNet(g);
        auto.toStart();
        while (!auto.isAtEnd() && auto.getStep() < step) {
            auto.stepForward(0);
        }
        setVisMode();
    }


    /**
     * This method creates panel with vis's controls.
     * @return panel with vis's controls.
     */
    public Component createControlsPane() {

        /**
         * Creates a panel.
         */
        panel = new Panel();

        /**
         * Sets the layout manager.
         */
        panel.setLayout(new BorderLayout());

        /**
         * Creates the panel to manage visualizer.
         */
        controlsPane = new AutoControlsPane(config, auto, forefather, true);

        /**
         * Creates the panel for editing net.
         */
        editPanel = new Panel();


/*-----------------------------------------------------------------------------------------------------------------------------------------------------*/
        if (config.getBoolean("button-ShowSaveLoad")) { 
            saveLoadButton = new HintedButton(config, "button-SaveLoad") {
                protected void click() {
                    saveLoadDialog.center();
                    StringBuffer buffer = new StringBuffer();
					
					/* Counts the number of arcs in the current net */
					int m = 0;
					for (int i = 0; i < data.n; i++) {
						for (int j = 0; j < data.n; j++) {
							if (data.g[i][j] > 0) {
								m++;
							}
						}
					}

                    buffer.append("/* ").append(
                            config.getParameter("NumberOfVerticesComment")
                    ).append(" */\n");

                    buffer.append("NumberOfVertices = ").append(data.n).append("\n\n");

                    buffer.append("/* ").append(
                        I18n.message(
                            config.getParameter("NumberOfArcsComment"), 
                            new Integer(data.n)
                        )
                    ).append(" */\n");

                    buffer.append("NumberOfArcs = ").append(m).append("\n\n");
                                                            
                    buffer.append("/* ").append(
                            config.getParameter("ArcsComment")
                    ).append(" */\n");

                    buffer.append("Arcs = \n");
                    for (int i = 0; i < data.n; i++) {
                    	for (int j = 0; j < data.n; j++) {
                        	if (data.g[i][j] > 0) {
                        		buffer.append(i + " " + j + " " + data.g[i][j]).append("\n");
                        	}
                        }
                    }

                    buffer.append("\n/* ").append(
                        config.getParameter("StepComment")
                    ).append(" */\n");
                    buffer.append("Step = ").append(auto.getStep());

                    saveLoadDialog.show(buffer.toString());
                }
            };
        }

        saveLoadDialog = new SaveLoadDialog(config, forefather) {
            public boolean load(String text) throws Exception {
                SmartTokenizer tokenizer = new SmartTokenizer(text, config);

                tokenizer.expect("NumberOfVertices");
                tokenizer.expect("=");
                int n = tokenizer.nextInt(2, 10);
                int[][] g = new int[n][n];

                tokenizer.expect("NumberOfArcs");
                tokenizer.expect("=");
                int m = tokenizer.nextInt(0, n * (n - 1) / 2);

                tokenizer.expect("Arcs");
                tokenizer.expect("=");
                int i, j, c;
                for (int k = 0; k < m; k++) {
					i = tokenizer.nextInt(0, n - 1);
					j = tokenizer.nextInt(0, n - 1);
					c = tokenizer.nextInt(1, config.getInteger("maxArcCapacity"));
					if (g[j][i] > 0 || g[i][j] > 0) {
						throw new Exception( I18n.message( config.getParameter( "TwoArcsExceptionMsg"), new Integer(tokenizer.lineno() ) ) );
					} 
                   	g[i][j] = c;
                }

                tokenizer.expect("Step");
                tokenizer.expect("=");
                int step = tokenizer.nextInt();

                tokenizer.expectEOF();

                rewind(n, g, step);

                return true;
            }
        };

/*-----------------------------------------------------------------------------------------------------------------------------------------------------*/


        /**
         * Creates the button to change visualizing mode to editing mode and vice versa.
         */
        String states[] = new String[2];
        states[0] = "multibutton-edit";
        states[1] = "multibutton-vis";
        modeButton = new MultiButton(config, states) {
            protected int click(int state) {
                if (state == 0) {
                    auto.toStart();
                    setEditMode();          
                    showNetwork(false);
                    return 1;
                } else {
                    setVisMode();
                    showNetwork(false);
                    return 0;
                }
            }
        };

        /**
         * Creates the button for randomizing the net.
         */
        HintedButton randomButton = new HintedButton(config, "random-button") {
            protected void click() {
                 auto.toStart();
                randomizeNet();
                showNetwork(false);
            }
        };

        /**
         * Create the panel to show number of the vertices.
         */
        nPanel = new SpinPanel(config, "nPanel") {
            protected void click(double value) {

                /**
                 * Sets an automata to the start state.
                 */
                while (!auto.isAtStart()) {
                    auto.stepBackward(1);
                }

                /**
                 * Gets number of the vertices and sets number to the sink according to this information.
                 */
                int oldn = data.n;
                data.n = getIntValue();
                data.t = data.n - 1;
                resizeNet(oldn);

                /**
                 * Sets number of vertices in net.
                 */
                net.setNumOfVertices(data.n);

                /**
                 * Shows a net.
                 */
                showNetwork(false);
                net.repaint();
            }
        };

        HintedButton clearButton = new HintedButton(config, "clear-button") {
            protected void click() {
                /**
                 * Sets an automata to start state.
                 */
                auto.toStart();

                /**
                 * Clear the net.
                 */
                clearNet();

                /**
                 * Shows an initial net without flow.
                 */
                showNetwork(false);
            }
        };

        bottomPanel = new Panel();
        bottomPanel.add(saveLoadButton);
        bottomPanel.add(modeButton);
        editPanel.add(clearButton);
        editPanel.add(nPanel);
        editPanel.add(randomButton);

        /**
         * Ands this panel to main panel.
         */
        panel.add(controlsPane, BorderLayout.NORTH);
        panel.add(editPanel, BorderLayout.CENTER);
        panel.add(bottomPanel, BorderLayout.SOUTH);
        return panel;


    }

    /**
     * Changes the size of the net.
     * @param oldn old size.
     */                                       	
    private void resizeNet(int oldn) {
        int oldg[][] =  new int[oldn][oldn];
        for (int i = 0; i < oldn; i++)
            for (int j = 0; j < oldn; j++)
                oldg[i][j] = data.g[i][j];
        allocateMem();
        if (oldn < data.n) {
            for (int i = 0; i < oldn; i++) {
                for (int j = 0; j < oldn; j++) {
                    if (i != oldn - 1 && j != oldn - 1) {
                        data.g[i][j] = oldg[i][j];
                     } else if (i == oldn - 1) {
                        data.g[data.n - 1][j] = oldg[i][j];
                     } else {
                        data.g[i][data.n - 1] = oldg[i][j];
                     }
                }
            }
        } else {
            for (int i = 0; i < oldn; i++) {
                for (int j = 0; j < oldn; j++) {
                    if (i == oldn - 2 || j == oldn - 2 || oldg[i][j] == 0) continue;
                    if (i != oldn - 1 && j != oldn - 1) {
                        data.g[i][j] = oldg[i][j];
                    } else if (i == oldn - 1) {
                        data.g[data.n - 1][j] = oldg[oldn - 1][j];
                    } else if (j == oldn - 1) {
                        data.g[i][data.n - 1] = oldg[i][oldn - 1];
                    }
                }
            }
        }
    }

    /**
     * Save the net.
     */
    public String saveNet() {   	
        NetRecord netRecord = new NetRecord(data.n, data.g);
        return netRecord.writeNet();
    }

    /**
     * Load the net.
     */
    public void loadNet(String s) {    	
 		NetRecord netRecord = new NetRecord(s, true);																
 		if (netRecord.n == 0) {
 			return;
 		}
        clearNet();
        data.n = netRecord.n;
        data.t = data.n - 1;
        net.setNumOfVertices(data.n);
        allocateMem();
        setNet(netRecord.g);
        auto.toStart();
        if (netRecord.step > 0) {
            while (netRecord.step != auto.getStep()) {
                auto.stepForward(0);
            }
            setVisMode();
        }
    }


    //     (data.g)
    private void setNet(int g[][]) {
        for (int i = 0; i < data.n; i++)
            for (int j = 0; j < data.n; j++)
                data.g[i][j] = g[i][j];
    }

    //     
    private void setEditMode() {
        modeButton.setState(1);
        controlsPane.setVisible(false);
        editPanel.setVisible(true);
        if (!panel.isValid()) {
            panel.validate();
        }
        modeButton.requestFocus();
        activateNetEditor();
    }

    //    
    private void setVisMode() {
        modeButton.setState(0);
        editPanel.setVisible(false);
        controlsPane.setVisible(true);
        if (!panel.isValid()) {
            panel.validate();
        }
        modeButton.requestFocus();
        deactivateNetEditor();
     }

    private void clearNet() {
        int i, j;
        int n = data.n;
        for (i = 0; i < n; i++)
            for (j = 0; j < n; j++) {
                data.g[i][j] = 0;
                data.f[i][j] = 0;
                data.dg[i][j] = 0;
                data.df[i][j] = 0;
                data.newArc[i][j] = false;
                data.newFlowArc[i][j] = false;
           }
        for (i = 0; i < n; i++) {
        data.dq = new int[n];
            data.p[i] = 0;
            data.pin[i] = 0;
            data.pout[i] = 0;
            data.del[i] = false;
            data.dst[i] = 0;
            data.q1[i] = 0;
            data.q2[i] = 0;
        data.q[i] = 0;
            data.diniz[i] = false;
            data.fl[i] = 0;
        data.dq[i] = 0;
        }
        data.pr = 0;
        data.pw = 0;
        data.dpw = 0;
        data.dpr = 0;
    }

    private void randomizeNet() {
        clearNet();
        RandomNetGenerator.setConfig(config);
        RandomNetGenerator.setMethod(RandomNetGenerator.ALL_VERTICES_METHOD);
        RandomNetGenerator.random(data.n, data.g);
    }

    /**
     * Invoked when state changed.
     */

    //     
    private void allocateMem() {
        int n = data.n;
        data.g = new int[n][n];
        data.f = new int[n][n];
        data.dg = new int[n][n];
        data.df = new int[n][n];
        data.p = new int[n];
        data.pin = new int[n];
        data.pout = new int[n];
        data.del = new boolean[n];
        data.diniz = new boolean[n];
        data.newArc = new boolean[n][n];
        data.newFlowArc = new boolean[n][n];
        data.fl = new int[n];
        data.q = new int[n];
        data.dq = new int[n];
        data.dst = new int[n];
        data.q1 = new int[n];
        data.q2 = new int[n];
        data.s = 0;
        data.pr = 0;
        data.pw = 0;
        data.dpw = 0;
        data.dpr = 0;

        //    data,    (   net   editor-)
        net.updateAppletData(data.g,  data.f);
        editor.updateAppletData(data.g);
    }

    /**
     * Creates an vis.
     */
    public DMKMVisualizer(VisualizerParameters visualizerParameters) {
        super(visualizerParameters);

        forefather = visualizerParameters.getForefather();
        auto = new DMKM(locale);
        data = auto.d;
        data.vis = this;

        /**
         * Creates an editor for the net.
         */
        editor = new NetEditor(config, this);

        /**
         * Create the net.
         */
        net = new Net(config, editor);

        createInterface(auto);

/*-----------------------------------------------------------------------------------------------------------------------------------------
        Frame frame = new Frame();
        saveDialog = new SaveLoadDialog(frame, this, config, SaveLoadDialog.SAVE);
        loadDialog = new SaveLoadDialog(frame, this, config, SaveLoadDialog.LOAD);
------------------------------------------------------------------------------------------------------------------------------------------*/

        lNets = new NetRecord[0];

        /**
         *  Loads some examples.
         */
        loadExampleNets();

        /**
         * Loads the first net from examples.
         */
        onLoad(1);

        /**
         * Sets the net to editor.
         */
        editor.setNet(net);

        /**
         * Sets number of vertices in net.
         */
        net.setNumOfVertices(data.n);

        /**
         * Adds net to the client pane.
         */
        clientPane.add(net);

        /**
         * Sets visualization mode.
         */
        setVisMode();

        showNetwork(false);

    }

    public void onLoad(int i) {
        if (i == -1) return;
        clearNet();
        NetRecord lNet = lNets[i];
        nPanel.setValue(lNet.n);
        data.n = lNet.n;
        data.t = data.n - 1;
        net.setNumOfVertices(data.n);
        allocateMem();
        setNet(lNet.g);
        auto.toStart();
        if (lNet.step > 0) {
            while (lNet.step != auto.getStep()) {
                auto.stepForward(0);
            }
            setVisMode();
        }
    }

    /**
     * Sets an editor to active mode.
     */
    private void activateNetEditor() {
        editor.setActive(data.g, net.getVertexLayouter());
        net.addMouseListener(editor);
        net.addMouseMotionListener(editor);
    }

    /**
     * Sets an editor to passive mode.
     */
    private void deactivateNetEditor() {
        editor.setPassive();
        net.removeMouseListener(editor);
        net.removeMouseMotionListener(editor);
    }

    /**
     * Invoked when net changed via NetEditor.
     */
    public void onNetChanged() {
        showNetwork(false);
        net.repaint();
    }

    /**
     * Invoked when arc dragged in edit mode.
     */
    public void onArcDragged() {
        net.repaint();
    }

    /**
     * Invoked when menu PopUped in edit mode.
     */
    public void onMenuPopUp() {
        showNetwork(false);
        net.repaint();
    }

    /**
     * Load some examples.
     */
    private void loadExampleNets() {
        int num = config.getInteger("example-num");
        String strNet, netName;
        for (int i = 0; i < num; i++) {
            strNet = config.getParameter("example-net" + (i + 1));
            netName =  config.getParameter("example-net" + (i + 1) + "-name");;
            if (!strNet.equals("")) {

                int len = lNets.length;
                NetRecord temp[] = new NetRecord[len];
                for (int j = 0; j < len; j++)
                    temp[j] = lNets[j];
                lNets = new NetRecord[len + 1];
                for (int j = 0; j < len; j++)
                    lNets[j] = temp[j];
                lNets[len] = new NetRecord(strNet);
/*-------------------------------------------------------------------------------------------------------------------------------------
                saveDialog.getList().add(netName);
                loadDialog.getList().add(netName);
---------------------------------------------------------------------------------------------------------------------------------------*/
            }
        }
    }

    /**
     *  Invoked when client pane reshaped.
     */
    protected void layoutClientPane(int width, int height) {
        net.setSize(width, height);
        net.paint();
    }

    /**
     * Paints network in current step.
     * @param isFlow if flow need to show.
     */
    public void showNetwork(boolean isFlow) {
        net.setNet(data.g);
        net.setFlow(data.f, isFlow);
        net.setPotentials(false);
        net.paint();
    }

    /**
     * Paints diniz network in current step.
     * @param isFlow if flow need to show.
     */
    public void showDiniz(boolean isFlow) {
        net.setNet(data.dg, data.del);
        net.setFlow(data.df, isFlow, data.newFlowArc);
        net.setPotentials(false);
        net.paint();
   }

    /**
     * Paints network with vertices potentials in current step.
     */
    public void showPotentials() {
        net.setNet(data.dg, data.del);
        net.setFlow(data.f, false);
        net.setPotentials(data.p);
        net.paint();
    }

    /**
     * Paints network after removing vertices with zero potential.
     * @param isFlow
     */
    public void showDelete(boolean isFlow) {
        net.setNet(data.dg, data.del);
        net.setFlow(data.df, isFlow);
        net.setPotentials(data.p);
        net.paint();
    }

    /**
     * Paints network which contains selected vertex with minimum potential.
     * @param isFlow
     */
    public void showMinv(boolean isFlow) {
        net.setNet(data.dg, data.del);
        net.setFlow(data.df, isFlow);
        net.setPotentials(data.p, data.minv);
        net.paint();
    }

    /**
     * Paints network after pushing flow forward.
     */
    public void showForward() {
        net.setNet(data.dg, data.del);
        net.setFlow(data.df, true, data.newArc);
        net.setPotentials(data.p, data.minv);
        net.paint();
    }

    /**
     * Paints network after pushing flow backward.
     */
    public void showBackward() {
        net.setNet(data.dg, data.del);
        net.setFlow(data.df, true, data.newArc);
        net.setPotentials(data.p, data.minv);
        net.paint();
    }

}

