package ru.ifmo.vizi.dmkm.net;

import ru.ifmo.vizi.base.Configuration;

/** This class ranmomizes the net.
 * @author Yuri Bedny
 * @version 0.0
 */
public class RandomNetGenerator {

    public static final int DEFAULT_MAX_ARC_CAPACITY = 9;

    public static final int ST_PATHS_METHOD = 0;

    public static final int ALL_VERTICES_METHOD = 1;

    public static final double DEFAULT_FROM_SOURCE_PROBABILITY = 0.5;

    public static final double DEFAULT_TO_SINK_PROBABILITY = 0.5;

    public static final int DEFAULT_MIN_FROM_SOURCE_CAPACITY = 5;

    public static final int DEFAULT_PATHS_COUNT = 5;

    private static int method = ALL_VERTICES_METHOD;

    private static double fromSourceProbability = DEFAULT_FROM_SOURCE_PROBABILITY;

    private static double toSinkProbability = DEFAULT_TO_SINK_PROBABILITY;

    private static int minFromSourceCapacity = DEFAULT_MIN_FROM_SOURCE_CAPACITY;

    private static int pathsCount = DEFAULT_PATHS_COUNT;

    private static Configuration config;

    private static int maxArcCapacity;

    private static boolean used[];

    private static boolean endFlag;

    private static int[][] g;

    private static int n;

    /**
     * This class can't be instanceated.
     */
    private RandomNetGenerator(){
    }

    /**
     * Sets the existence probability for arcs which starts at source.
     * @param p probability to set.
     */
    public static void setFromSourceProbability(double p) {
        fromSourceProbability = p;
    }

    /**
     * Sets vizi's configuration.
     * @param cfg configuration to set.
     */
    public static void setConfig(Configuration cfg) {
    	config = cfg;
    }

    /**
     * Whether add an arc between vertices i and j.
     * @param i the first vertex.
     * @param j the second vertex.
     * @return Whether add an arc between vertices i and j.
     */
    private static boolean bad(int i, int j) {
        boolean topi = i > 0 && i < (n + 1)/2;
        boolean topj = j > 0 && j < (n + 1)/2;

        if (i == 0 || j == 0 || i == n - 1 || j == n - 1) {
            return false;
        }
        if (topi ^ topj) {
            i = i % (n / 2);
            j = j % (n / 2);
            if (Math.abs(i - j) > 2) {
                return true;
            }
            return false;
        }

        if (Math.abs(i - j) != 1) {
            return true;
        }

        return false;
    }

    /**
     * Creates an arc between vertices i and j.
     * @param i the first vertex.
     * @param t the second vertex.
     */
    private static void go(int i, int t) {
        used[i] = true;
        if (i == t) {
            endFlag = true;
        }
        if (endFlag) return;
        int j;
        do {
            if (t == n - 1) {
                j = 1 + Math.round((int) (Math.random() * (n - 1)));
            } else {
                j = Math.round((int)(Math.random() * (n - 1)));
            }
        } while ( (i == j) || (t == n - 1 && (g[j][i] > 0 || bad(j, i))) || (t == 0 && (g[i][j] > 0 || bad(i, j))));
        if (t == n - 1) {
            g[i][j] = 1 + (int)(Math.random() * maxArcCapacity);
        } else {
            g[j][i] = 1 + (int)(Math.random() * maxArcCapacity);
        }
        go(j, t);
    }

    /**
     * Sets the number of paths between source and sink to create.
     * @param cnt number of paths between source and sink to create.
     */
    public static void setPathsCount(int cnt) {
        pathsCount = cnt;
    }

    /**
     * Sets the net generation method.
     * @param m the net generation method.
     */
    public static void setMethod(int m) {
        method = m;
    }

    /**
     * Randomize the net.
     * @param num number of vertices in net.
     * @param net net.
     */
    public static void random(int num, int net[][]) {
    	if (config != null) {
            maxArcCapacity = config.getInteger("maxArcCapacity");
        } else {
            maxArcCapacity = DEFAULT_MAX_ARC_CAPACITY;
        }
        g = net;
        n = num;
        used = new boolean[n];
        for (int i = 0; i < n; i++)
            used[i] = false;
        if (method == ST_PATHS_METHOD) {
            for (int i = 0; i < pathsCount; i++) {
                endFlag = false;
                go(0, n - 1);
            }
        } else if (method == ALL_VERTICES_METHOD) {
            for (int i = 1; i < n - 1; i++) {
                if (!used[i]) {
                    endFlag = false;
                    go(i, 0);
                    endFlag = false;
                    go(i, n - 1);
                }
            }

            int[] fromSrc = new int[]{1, (n + 1) / 2 + 1};
            int[] toDst = new int[]{(n + 1) / 2, n - 2};

            if (n >= 4) {
                for (int i = 1; i < fromSrc.length; i++) {
                    if (Math.random() < fromSourceProbability) {
                        g[0][fromSrc[i]] = (int) (minFromSourceCapacity + Math.random() * (maxArcCapacity - minFromSourceCapacity + 1));
                    }
                }
                for (int i = 0; i < toDst.length; i++) {
                    if (Math.random() < toSinkProbability) {
                        g[toDst[i]][n - 1] = (int) (1 + Math.random() * maxArcCapacity);
                    }
                }
            } else if (Math.random() < fromSourceProbability) {
                g[0][n - 1] = (int) (minFromSourceCapacity + Math.random() * (maxArcCapacity - minFromSourceCapacity + 1));
            }
        }
    }
}
