package ru.ifmo.vizi.dmkm.ui;

import ru.ifmo.vizi.base.widgets.Shape;
import ru.ifmo.vizi.base.widgets.ShapeStyle;

import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * The button in editor menu.
 */
public class Button extends Shape {

    /**
     * Whether button is pressed.
     */
    private boolean isPressed = false;

    /**
     * Whether button is in "down" state.
     */
    private boolean isDown = false;

    /**
     * An adapter.
     */
    private MouseAdapter mouseAdapter;

    /**
     * Creates the button with specified style set.
     * @param styles style set for the button.
     */
    public Button(ShapeStyle[] styles) {
        this(styles, "");
    }

    /**
     * Creates the button with specified style set and message.
     * @param styles style set for the button.
     * @param msg button's message.
     */
    public Button(ShapeStyle[] styles, String msg) {
        super(styles, msg);
        final Button button = this;
        mouseAdapter = new MouseAdapter(){
            public void mousePressed(MouseEvent me) {
                isPressed = true;
                isDown = true;
                repaint();
            }
            public void mouseExited(MouseEvent me) {
                isDown = false;
                repaint();
            }
            public void mouseReleased(MouseEvent me) {
                if (isDown) {
                    button.click();
                }
                isPressed = false;
                if (me.getSource() == button) {
                    isDown = false;
                    repaint();
                }
            }
            public void mouseEntered(MouseEvent me) {
                if (isPressed) {
                    isDown = true;
                    repaint();
                }
            }
        };
        addMouseListener(mouseAdapter);
    }

    public Dimension fit(Dimension dim) {
        return dim;
    }

    /**
     * Paints the button.
     * @param g Graphics to paint.
     */
    public void paint(Graphics g) {
        int width = getSize().width - 1;
        int height = getSize().height - 1;
        if (look.getFillStatus(style)) {
            g.setColor(look.getFillColor(style));
            g.fillRect(0, 0, width, height);
        }
        if (look.getBorderStatus(style)) {
            g.setColor(look.getBorderColor(style));
            g.drawRect(0, 0, width, height);
        }
        if (isDown) g.setColor(Color.gray); else g.setColor(Color.white);
        g.drawLine(0, 0, width, 0);
        g.drawLine(0, 0, 0, height);
        if (isDown) g.setColor(Color.lightGray); else g.setColor(Color.white);
        g.drawLine(1, 1, width - 1, 1);
        g.drawLine(1, 1, 1, height - 1);
        if (isDown) g.setColor(Color.white); else g.setColor(Color.gray);
        g.drawLine(0, height, width, height);
        g.drawLine(width, 0, width, height);
        if (isDown) g.setColor(Color.white); else g.setColor(Color.lightGray);
        g.drawLine(1, height - 1, width - 1, height - 1);
        g.drawLine(width - 1, 1, width - 1, height - 1);
        super.paint(g);
    }

    /**
     * Enables / disables the button.
     * @param enabled Whether button is enabled.
     */
    public void setEnabled(boolean enabled) {
        if (enabled) {
            removeMouseListener(mouseAdapter);
            addMouseListener(mouseAdapter);
            look.setTextColor(style.getTextColor());
        } else {
            removeMouseListener(mouseAdapter);
            look.setTextColor(Color.lightGray);
        }
        repaint();
    }

    protected void click() {

    }

}
