package ru.ifmo.vizi.dmkm.ui;

import ru.ifmo.vizi.base.Configuration;
import ru.ifmo.vizi.base.ui.HintedButton;
import ru.ifmo.vizi.dmkm.DMKMVisualizer;

import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * The dialog to save and load net states.
 */
public class SaveLoadDialog extends Dialog implements ItemListener {

    /**
     * Type of the dialog.
     */
    public static final int SAVE = 0;
    public static final int LOAD = 1;

    /**
     * Controls parameters.
     */
    private final int TEXT_LENGTH = 50;
    private final int NETS_IN_LIST = 10;

    /**
     * The controls.
     */
    private HintedButton saveLoad;
    private HintedButton cancel;
    private TextField netName;
    private List netList;

    /**
     * Creates a dialog with specified owner, visualizer, configuration and mode.
     * @param owner the owner window for the dialog.
     * @param applet visualizer.
     * @param config vizi's configuration.
     * @param mode dialog's mode.
     */
    public SaveLoadDialog(Frame owner, final DMKMVisualizer applet, Configuration config, int mode) {
        super(owner, true);
        String title = "";
        if (mode == SAVE) {
            title = config.getParameter("netDialog-save");
            saveLoad = new HintedButton(config, "netDialog-saveButton") {
                protected void click() {
                    onCancel();
                    netList.add(netName.getText());
//                    applet.onSave(netName.getText());
                }
            };

        } else if (mode == LOAD) {
            title = config.getParameter("netDialog-load");
            saveLoad = new HintedButton(config, "netDialog-loadButton") {
                protected void click() {
                    onCancel();
                    int ind = -1;
                    String items[] = netList.getItems();
                    for (int i = 0; i < items.length; i++) {
                        if (items[i].equals(netName.getText())) {
                            ind = i;
                        }
                    }
                    applet.onLoad(ind);
                }
            };

            saveLoad.repaint();
        }


        setTitle(title);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                setVisible(false);
            }
        });

        setLayout(new FlowLayout(10));
        Panel panel = new Panel(new BorderLayout(10, 10));
        Panel okCancelContainerPanel = new Panel(new BorderLayout());
        Panel okCancelPanel = new Panel(new BorderLayout(5, 0));
        Panel bottomPanel = new Panel(new BorderLayout());
        Panel topPanel = new Panel(new BorderLayout());

        netList = new List(NETS_IN_LIST);

        netName = new TextField(TEXT_LENGTH);
        netList.addItemListener(this);

        cancel = new HintedButton(config, "netDialog-cancelButton") {
            protected void click() {
                onCancel();
            }
        };

        Label namesLabel = new Label(config.getParameter("netDialog-listLabel"));

        bottomPanel.add(new Label(config.getParameter("netDialog-recordNameCaption")), BorderLayout.WEST);
        bottomPanel.add(netName, BorderLayout.CENTER);

        okCancelPanel.add(saveLoad, BorderLayout.WEST);
        okCancelPanel.add(cancel, BorderLayout.EAST);
        okCancelContainerPanel.add(okCancelPanel, BorderLayout.EAST);

        topPanel.add(namesLabel, BorderLayout.NORTH);
        topPanel.add(netList, BorderLayout.SOUTH);
        panel.add(bottomPanel, BorderLayout.CENTER);
        panel.add(okCancelContainerPanel, BorderLayout.SOUTH);
        panel.add(topPanel, BorderLayout.NORTH);
        add(panel);
        setResizable(false);
        setLocation(100, 100);
        pack();
    }

    /**
     * Invoked when cancel button was pressed.
     */
    private void onCancel() {
        setVisible(false);
    }

    public List getList() {
        return netList;
    }

    /**
     * Invoked when item's state changed.
     */
    public void itemStateChanged(ItemEvent e) {
        netName.setText(netList.getSelectedItem());
    }
}
