package ru.ifmo.vizi.dmkm.widgets;

import ru.ifmo.vizi.base.widgets.ShapeStyle;

import java.awt.*;

/**
 * The arc of the net.
 *
 * @author Yuri Bedny
 * @version $Id$
 */

public class Arc extends Arrow {

    private Point p1;

    private Point p2;

    private int capacity;

    private int flow;

    private boolean isFlow;

    private boolean isCapacity;

    /**
     * Creates a new arc.
     *
     * @param styleSet set of styles for this arc.
     * @param diam diameter of thirst and secondPoint arc's ends.
     * @param p1 thirst arc's point to draw.
     * @param p2 secondPoint arc's point to draw.
     * @param c arrow's capacity.
     * @param f arrow's flow.
     */
    public Arc(ShapeStyle[] styleSet, int diam, Point p1, Point p2, int c, int f) {
        this(styleSet, diam, diam, p1, p2, c, f);
    }

    /**
     * Creates a new arrow.
     *
     * @param styleSet set of styles for this arrow.
     * @param diam1 diameter of thirst arrow's vertex.
     * @param diam2 diameter of secondPoint arrow's vertex.
     * @param p1 thirst arrow's point to draw.
     * @param p2 secondPoint arrow's point to draw.
     * @param c arrow's capacity.
     * @param f arrow's flow.
     */
    public Arc(ShapeStyle[] styleSet, int diam1, int diam2, Point p1, Point p2, int c, int f) {
        super(styleSet, p1, p2);
        double l = dist(p1, p2);
        int x1 = p1.x - (int)((p1.x - p2.x) * diam1 / 2 / l);
        int y1 = p1.y - (int)((p1.y - p2.y) * diam1 / 2 / l);
        int x2 = p2.x + (int)((p1.x - p2.x) * diam2 / 2 / l);
        int y2 = p2.y + (int)((p1.y - p2.y) * diam2 / 2 / l);
        startPoint = new Point(x1, y1);
        secondPoint = new Point(x2, y2);
        isFlow = true;
        isCapacity = true;
        setLocation(0, 0);
        this.p1 = p1;
        this.p2 = p2;
        capacity = c;
        flow = f;
    }

    /**
     * Creates a new arrow.
     * @param styleSet set of styles for this arrow.
     */
    public Arc(ShapeStyle[] styleSet) {
        this(styleSet, 0, 0, new Point(0, 0), new Point(0, 0));
    }

    /**
     * Creates a new arrow.
     * @param styleSet Set of styles for this arrow.
     * @param diam1 Diameter of thirst arrow's vertex.
     * @param diam2 Diameter of secondPoint arrow's vertex.
     * @param p1 Thirst arrow's point to draw.
     * @param p2 Second arrow's point to draw.
     */
    public Arc(ShapeStyle[] styleSet, int diam1, int diam2, Point p1, Point p2) {
        this(styleSet, diam1, diam2, p1, p2, 0, 0);
        isFlow = false;
        isCapacity = false;
    }

    /**
     * Creates a new arrow.
     * @param styleSet set of styles for this arrow.
     * @param diam diameter of thirst and secondPoint arrow's vertices.
     * @param p1 thirst arrow's point to draw.
     * @param p2 secondPoint arrow's point to draw.
     * @param c arrow's capacity
     */
    public Arc(ShapeStyle[] styleSet, int diam, Point p1, Point p2, int c) {
        this(styleSet, diam, p1, p2, c, 0);
        isFlow = false;
    }

    /**
     * Returns minimum shape size to contain message with specified size.
     * @param size message size.
     * @return minimum shape size.
     */
    protected Dimension fit(Dimension size) {
        return new Dimension(
                Math.round(size.width),
                Math.round(size.height)
        );
    }

    private double dist(Point p1, Point p2) {
        double dx = p2.x - p1.x;
        double dy = p2.y - p1.y;
        return Math.sqrt(dx * dx + dy * dy);
    }

    /**
     * Returns location for (flow/capacity) message.
     * @param circleDiam diameter of circle which contains message (flow/capacity).
     * @return message location.
     */
    public Point getTextLocation(int circleDiam) {
        int x1 = p1.x;
        int y1 = p1.y;
        int x2 = p2.x;
        int y2 = p2.y;
        int tx = (x1 + (x2 - x1) / 3);
        int ty = (y1 + (y2 - y1) / 3);
        return new Point(tx - circleDiam / 2, ty - circleDiam / 2);
    }

    /**
     * Returns (flow/capacity) message.
     * @return (flow/capacity) message.
     */
    public String getText() {
        if (isFlow && isCapacity)
            return flow + "/" + capacity;
        else if (isCapacity)
            return Integer.toString(capacity);
        else
            return "";
    }

}
