package ru.ifmo.vizi.dmkm.widgets;

import ru.ifmo.vizi.base.widgets.Shape;
import ru.ifmo.vizi.base.widgets.ShapeStyle;

import java.awt.*;

/**
 * Rectangular shape.
 * @author Yuri Bedny
 * @version 0.0
 */
public class Rect extends Shape {

    private int width;
    private int height;

    /**
     * Creates a new rectangle with specifed style set and size.
     * @param styleSet shape's style set.
     * @param width width.
     * @param height height.
     */
    public Rect(ShapeStyle[] styleSet, int width, int height) {
        this(styleSet, width, height, "");
    }

    /**
     * Creates a new rectangle with specifed style set, message and size.
     * @param styleSet shape's style set.
     * @param width width.
     * @param height height.
     * @param message message to show.
     */
    public Rect(ShapeStyle[] styleSet, int width, int height, String message) {
        super(styleSet, message);
        this.width = width;
        this.height = height;
        setSize(width + 1, height + 1);
    }

    /**
     * Returns minimum shape size to contain message with specified size.
     * @param size message size.
     * @return minimum shape size.
     */
    protected Dimension fit(Dimension size) {
        return new Dimension(
                Math.round(size.width),
                Math.round(size.height)
        );
    }

    /**
     * Paints this component.
     * @param g graphics context for painting.
     */
    public void paint(Graphics g) {
/*          System.out.println("FOR s = " + g.getFontMetrics().getLineMetrics("s", g).getHeight());
        System.out.println("FOR A = " + g.getFontMetrics().getLineMetrics("A", g).getHeight());
        System.out.println();*/

    
        if (look.getFillStatus(style)) {
            g.setColor(look.getFillColor(style));
            g.fillRect(0, 0, width, height);
        }
        if (look.getBorderStatus(style)) {
            g.setColor(look.getBorderColor(style));
            g.drawRect(0, 0, width, height);
        }
        super.paint(g);
    }
}
